@@ -57,7 +57,10 @@ class UsersRestServlet(RestServlet):
             raise SynapseError(400, "Can only users a local user")
 
         ret = await self.store.get_users()
-
+        for user in ret:
+            profile = await self.store.get_profileinfo(user["name"])
+            user["displayname"] = profile.display_name
+            user["avatar_url"] = profile.avatar_url
         return 200, ret
 
 
@@ -611,6 +614,10 @@ class SearchUsersRestServlet(RestServlet):
         logger.info("term: %s ", term)
 
         ret = await self.handlers.store.search_users(term)
+        for user in ret:
+            profile = await self.store.get_profileinfo(user["name"])
+            user["displayname"] = profile.display_name
+            user["avatar_url"] = profile.avatar_url
         return 200, ret
 
 
